@@ -63,202 +63,6 @@ dump_hex (const unsigned char *p, size_t n)
 }
 #endif
 
-
-static  void
-parse_skip (unsigned char const **buf, size_t *len, struct tag_info *ti)
-{
-  if (ti->length)
-    {
-      assert (ti->length <= *len);
-      *len -= ti->length;
-      *buf += ti->length;
-    }
-}
-
-static gpg_error_t
-parse_sequence (unsigned char const **buf, size_t *len, struct tag_info *ti)
-{
-  gpg_error_t err;
-
-  err = _ksba_ber_parse_tl (buf, len, ti);
-  if (err)
-    ;
-  else if (!(ti->class == CLASS_UNIVERSAL && ti->tag == TYPE_SEQUENCE
-             && ti->is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (ti->length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-  return err;
-}
-
-static gpg_error_t
-parse_enumerated (unsigned char const **buf, size_t *len, struct tag_info *ti,
-                  size_t maxlen)
-{
-  gpg_error_t err;
-
-  err = _ksba_ber_parse_tl (buf, len, ti);
-  if (err)
-     ;
-  else if (!(ti->class == CLASS_UNIVERSAL && ti->tag == TYPE_ENUMERATED
-             && !ti->is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (!ti->length)
-    err = gpg_error (GPG_ERR_TOO_SHORT);
-  else if (maxlen && ti->length > maxlen)
-    err = gpg_error (GPG_ERR_TOO_LARGE);
-  else if (ti->length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-
-  return err;
-}
-
-static gpg_error_t
-parse_integer (unsigned char const **buf, size_t *len, struct tag_info *ti)
-{
-  gpg_error_t err;
-
-  err = _ksba_ber_parse_tl (buf, len, ti);
-  if (err)
-     ;
-  else if (!(ti->class == CLASS_UNIVERSAL && ti->tag == TYPE_INTEGER
-             && !ti->is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (!ti->length)
-    err = gpg_error (GPG_ERR_TOO_SHORT);
-  else if (ti->length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-
-  return err;
-}
-
-static gpg_error_t
-parse_octet_string (unsigned char const **buf, size_t *len, struct tag_info *ti)
-{
-  gpg_error_t err;
-
-  err= _ksba_ber_parse_tl (buf, len, ti);
-  if (err)
-    ;
-  else if (!(ti->class == CLASS_UNIVERSAL && ti->tag == TYPE_OCTET_STRING
-             && !ti->is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (!ti->length)
-    err = gpg_error (GPG_ERR_TOO_SHORT);
-  else if (ti->length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-
-  return err;
-}
-
-
-/* Note that R_BOOL will only be set if a value has been given. Thus
-   the caller should set it to the default value prior to calling this
-   function.  Obviously no call to parse_skip is required after
-   calling this function. */
-static gpg_error_t
-parse_optional_boolean (unsigned char const **buf, size_t *len, int *r_bool)
-{
-  gpg_error_t err;
-  struct tag_info ti;
-
-  err = _ksba_ber_parse_tl (buf, len, &ti);
-  if (err)
-    ;
-  else if (!ti.length)
-    err = gpg_error (GPG_ERR_TOO_SHORT);
-  else if (ti.length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-  else if (ti.class == CLASS_UNIVERSAL && ti.tag == TYPE_BOOLEAN
-           && !ti.is_constructed)
-    {
-      if (ti.length != 1)
-        err = gpg_error (GPG_ERR_BAD_BER);
-      *r_bool = !!**buf;
-      parse_skip (buf, len, &ti);
-    }
-  else
-    { /* Undo the read. */
-      *buf -= ti.nhdr;
-      *len += ti.nhdr;
-    }
-
-  return err;
-}
-
-
-
-static gpg_error_t
-parse_object_id_into_str (unsigned char const **buf, size_t *len, char **oid)
-{
-  struct tag_info ti;
-  gpg_error_t err;
-
-  *oid = NULL;
-  err = _ksba_ber_parse_tl (buf, len, &ti);
-  if (err)
-    ;
-  else if (!(ti.class == CLASS_UNIVERSAL && ti.tag == TYPE_OBJECT_ID
-                && !ti.is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (!ti.length)
-    err = gpg_error (GPG_ERR_TOO_SHORT);
-  else if (ti.length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-  else if (!(*oid = ksba_oid_to_str (*buf, ti.length)))
-    err = gpg_error_from_syserror ();
-  else
-    {
-      *buf += ti.length;
-      *len -= ti.length;
-    }
-  return err;
-}
-
-
-static gpg_error_t
-parse_asntime_into_isotime (unsigned char const **buf, size_t *len,
-                            ksba_isotime_t isotime)
-{
-  struct tag_info ti;
-  gpg_error_t err;
-
-  err = _ksba_ber_parse_tl (buf, len, &ti);
-  if (err)
-    ;
-  else if ( !(ti.class == CLASS_UNIVERSAL
-              && (ti.tag == TYPE_UTC_TIME || ti.tag == TYPE_GENERALIZED_TIME)
-              && !ti.is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (ti.length > *len)
-    err = gpg_error (GPG_ERR_INV_BER);
-  else if (!(err = _ksba_asntime_to_iso (*buf, ti.length,
-                                         ti.tag == TYPE_UTC_TIME, isotime)))
-    parse_skip (buf, len, &ti);
-
-  return err;
-}
-
-
-static gpg_error_t
-parse_context_tag (unsigned char const **buf, size_t *len, struct tag_info *ti,
-                   int tag)
-{
-  gpg_error_t err;
-
-  err = _ksba_ber_parse_tl (buf, len, ti);
-  if (err)
-    ;
-  else if (!(ti->class == CLASS_CONTEXT && ti->tag == tag
-	     && ti->is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (ti->length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-
-  return err;
-}
-
-
 
 /* Create a new OCSP object and retrun it in R_OCSP.  Return 0 on
    success or an error code.
@@ -474,7 +278,7 @@ write_request_extensions (ksba_ocsp_t ocsp, ksba_writer_t wout)
   if (err)
     goto leave;
 
-  /* Write OID and and nonce.  */
+  /* Write OID and nonce.  */
   err = ksba_oid_from_str (oidstr_ocsp_nonce, &buf, &buflen);
   if (err)
     goto leave;
@@ -554,6 +358,7 @@ ksba_ocsp_prepare_request (ksba_ocsp_t ocsp)
   unsigned char *p;
   const unsigned char *der;
   size_t derlen;
+  struct tag_info ti;
   ksba_writer_t w1 = NULL;
   ksba_writer_t w2 = NULL;
   ksba_writer_t w3 = NULL;
@@ -615,22 +420,28 @@ ksba_ocsp_prepare_request (ksba_ocsp_t ocsp)
 
       /* Write the serialNumber of the certificate to be checked. */
       err = _ksba_cert_get_serial_ptr (ri->cert, &der, &derlen);
-      if (!err)
-        err = _ksba_ber_write_tl (w1, TYPE_INTEGER, CLASS_UNIVERSAL, 0, derlen);
       if (!err)
         err = ksba_writer_write (w1, der, derlen);
       if (err)
         goto leave;
-      xfree (ri->serialno);
-      ri->serialno = xtrymalloc (derlen);
-      if (!ri->serialno)
-        {
-          err = gpg_error_from_syserror ();
-          goto leave;
-        }
-      memcpy (ri->serialno, der, derlen);
-      ri->serialnolen = derlen;
+      /* Store the integer value.  */
+      {
+        const unsigned char *tmpder = der;
+        size_t tmpderlen = derlen;
 
+        err = parse_integer (&tmpder, &tmpderlen, &ti);
+        if (err)
+          goto leave;
+        xfree (ri->serialno);
+        ri->serialno = xtrymalloc (tmpderlen);
+        if (!ri->serialno)
+          {
+            err = gpg_error_from_syserror ();
+            goto leave;
+          }
+        memcpy (ri->serialno, tmpder, tmpderlen);
+        ri->serialnolen = tmpderlen;
+      }
 
       /* Now write it out as a sequence to the outer certID object. */
       p = ksba_writer_snatch_mem (w1, &derlen);
@@ -909,8 +720,6 @@ parse_response_extensions (ksba_ocsp_t ocsp,
           if (ocsp->noncelen != ti.length
               || memcmp (ocsp->nonce, data, ti.length))
             ocsp->bad_nonce = 1;
-          else
-            ocsp->good_nonce = 1;
         }
       ex = xtrymalloc (sizeof *ex + strlen (oid) + ti.length);
       if (!ex)
@@ -1151,7 +960,7 @@ parse_single_response (ksba_ocsp_t ocsp,
   assert (n <= *datalen);
   *data += n;
   *datalen -= n;
-  /*   fprintf (stderr, "algorithmIdentifier is `%s'\n", oid); */
+  /* gpgrt_log_debug ("algorithmIdentifier is `%s'\n", oid); */
   look_for_request = !strcmp (oid, oidstr_sha1);
   xfree (oid);
 
@@ -1192,10 +1001,10 @@ parse_single_response (ksba_ocsp_t ocsp,
       for (request_item = ocsp->requestlist;
            request_item; request_item = request_item->next)
         if (!memcmp (request_item->issuer_name_hash, name_hash, 20)
-             && !memcmp (request_item->issuer_key_hash, key_hash, 20)
-             && request_item->serialnolen == serialnolen
+            && !memcmp (request_item->issuer_key_hash, key_hash, 20)
+            && request_item->serialnolen == serialnolen
             && !memcmp (request_item->serialno, serialno, serialnolen))
-          break; /* Got it. */
+            break; /* Got it. */
     }
 
 
@@ -1301,9 +1110,8 @@ parse_single_response (ksba_ocsp_t ocsp,
   err = parse_asntime_into_isotime (data, datalen, this_update);
   if (err)
     return err;
-/*   fprintf (stderr, "thisUpdate=%s\n", this_update); */
   if (request_item)
-      _ksba_copy_time (request_item->this_update, this_update);
+    _ksba_copy_time (request_item->this_update, this_update);
 
   /* nextUpdate is optional. */
   if (*data >= endptr)
@@ -1319,7 +1127,6 @@ parse_single_response (ksba_ocsp_t ocsp,
       err = parse_asntime_into_isotime (data, datalen, next_update);
       if (err)
         return err;
-/*       fprintf (stderr, "nextUpdate=%s\n", next_update); */
       if (request_item)
         _ksba_copy_time (request_item->next_update, next_update);
     }
@@ -1632,7 +1439,6 @@ ksba_ocsp_parse_response (ksba_ocsp_t ocsp,
   ocsp->received_certs = NULL;
   ocsp->hash_length = 0;
   ocsp->bad_nonce = 0;
-  ocsp->good_nonce = 0;
   xfree (ocsp->responder_id.name);
   ocsp->responder_id.name = NULL;
   xfree (ocsp->responder_id.keyid);
@@ -1654,9 +1460,12 @@ ksba_ocsp_parse_response (ksba_ocsp_t ocsp,
   /* FIXME: find duplicates in the request list and set them to the
      same status. */
 
-  if (*response_status == KSBA_OCSP_RSPSTATUS_SUCCESS)
-    if (ocsp->bad_nonce || (ocsp->noncelen && !ocsp->good_nonce))
-      *response_status = KSBA_OCSP_RSPSTATUS_REPLAYED;
+  /* We used to assume that the server needs to return a nonce, but
+   * that is not true (see for example RFC-8954).  Thus we do not
+   * check the former good_nonce flag anymore.  */
+  if (*response_status == KSBA_OCSP_RSPSTATUS_SUCCESS
+      && ocsp->bad_nonce)
+    *response_status = KSBA_OCSP_RSPSTATUS_REPLAYED;
 
   return err;
 }
