@@ -24,13 +24,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
 #include <stdlib.h>
 #include <time.h>
 #include <ctype.h>
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 #ifdef HAVE_LANGINFO_H
 #include <langinfo.h>
 #endif
@@ -39,6 +42,10 @@
 #include "i18n.h"
 #include "gettime.h"
 
+#ifdef HAVE_W32_SYSTEM
+#include <windows.h>
+#endif
+
 #ifdef HAVE_UNSIGNED_TIME_T
 # define IS_INVALID_TIME_T(a) ((a) == (time_t)(-1))
 #else
@@ -57,9 +64,12 @@ static enum { NORMAL = 0, FROZEN, FUTURE, PAST } timemode;
 /* Wrapper for the time(3).  We use this here so we can fake the time
    for tests */
 time_t
-gnupg_get_time ()
+gnupg_get_time (void)
 {
   time_t current = time (NULL);
+  if (current == (time_t)(-1))
+    log_fatal ("time() failed\n");
+
   if (timemode == NORMAL)
     return current;
   else if (timemode == FROZEN)
@@ -99,22 +109,16 @@ void
 gnupg_get_isotime (gnupg_isotime_t timebuf)
 {
   time_t atime = gnupg_get_time ();
+  struct tm *tp;
+  struct tm tmbuf;
 
-  if (atime == (time_t)(-1))
+  tp = gnupg_gmtime (&atime, &tmbuf);
+  if (!tp)
     *timebuf = 0;
   else
-    {
-      struct tm *tp;
-      struct tm tmbuf;
-
-      tp = gnupg_gmtime (&atime, &tmbuf);
-      if (!tp)
-        *timebuf = 0;
-      else
-        snprintf (timebuf, 16, "%04d%02d%02dT%02d%02d%02d",
-                  1900 + tp->tm_year, tp->tm_mon+1, tp->tm_mday,
-                  tp->tm_hour, tp->tm_min, tp->tm_sec);
-    }
+    snprintf (timebuf, 16, "%04d%02d%02dT%02d%02d%02d",
+              1900 + tp->tm_year, tp->tm_mon+1, tp->tm_mday,
+              tp->tm_hour, tp->tm_min, tp->tm_sec);
 }
 
 
@@ -136,7 +140,7 @@ gnupg_set_time (time_t newtime, int freeze)
   else if (freeze)
     {
       timemode = FROZEN;
-      timewarp = current;
+      timewarp = newtime == (time_t)-1 ? current : newtime;
     }
   else if (newtime > current)
     {
@@ -164,9 +168,6 @@ u32
 make_timestamp (void)
 {
   time_t t = gnupg_get_time ();
-
-  if (t == (time_t)-1)
-    log_fatal ("gnupg_get_time() failed\n");
   return (u32)t;
 }
 
@@ -228,6 +229,8 @@ isotime_p (const char *string)
   for (s++, i=9; i < 15; i++, s++)
     if (!digitp (s))
       return 0;
+  if (*s == 'Z')
+    s++;
   if ( !(!*s || (isascii (*s) && isspace(*s)) || *s == ':' || *s == ','))
     return 0;  /* Wrong delimiter.  */
 
@@ -360,9 +363,10 @@ string2isotime (gnupg_isotime_t atime, const char *string)
 }
 
 
-/* Scan an ISO timestamp and return an Epoch based timestamp.  The only
-   supported format is "yyyymmddThhmmss" delimited by white space, nul, a
-   colon or a comma.  Returns (time_t)(-1) for an invalid string.  */
+/* Scan an ISO timestamp and return an Epoch based timestamp.  The
+   only supported format is "yyyymmddThhmmss[Z]" delimited by white
+   space, nul, a colon or a comma.  Returns (time_t)(-1) for an
+   invalid string.  */
 time_t
 isotime2epoch (const char *string)
 {
@@ -682,9 +686,9 @@ isotimestamp (u32 stamp)
 const char *
 asctimestamp (u32 stamp)
 {
-  static char buffer[50];
+  static char buffer[80];
 #if defined (HAVE_STRFTIME) && defined (HAVE_NL_LANGINFO)
-  static char fmt[50];
+  static char fmt[80];
 #endif
   struct tm *tp;
   time_t atime = stamp;
@@ -710,6 +714,32 @@ asctimestamp (u32 stamp)
      zone at all.  */
   strftime (buffer, DIM(buffer)-1, "%c", tp);
 # else
+#  if HAVE_W32_SYSTEM
+  {
+    static int done;
+
+    if (!done)
+      {
+        /* The locale names as used by Windows are in the form
+         * "German_Germany.1252" or "German_Austria.1252" with
+         * alternate names similar to Unix, e.g. "de-DE".  However
+         * that is the theory.  On current Windows and Mingw the
+         * alternate names do not work.  We would need a table to map
+         * them from the short names as provided by gpgrt to the long
+         * names and append some code page.  For now we use "" and
+         * take the locale from the user's system settings.  Thus the
+         * standard Unix envvars don't work for time and may mismatch
+         * with the string translations.  The new UCRT available since
+         * 2018 has a lot of additional support but that will for sure
+         * break other things.  We should move to ISO strings to get
+         * rid of such problems.  */
+        setlocale (LC_TIME, "");
+        done = 1;
+        /* log_debug ("LC_ALL  now '%s'\n", setlocale (LC_ALL, NULL)); */
+        /* log_debug ("LC_TIME now '%s'\n", setlocale (LC_TIME, NULL)); */
+      }
+  }
+#  endif
    /* FIXME: we should check whether the locale appends a " %Z" These
     * locales from glibc don't put the " %Z": fi_FI hr_HR ja_JP lt_LT
     * lv_LV POSIX ru_RU ru_SU sv_FI sv_SE zh_CN.  */
@@ -746,10 +776,10 @@ rfctimestamp (u32 stamp)
   if (!tp)
     return NULL;
   return xtryasprintf ("%.3s, %02d %.3s %04d %02d:%02d:%02d +0000",
-                       ("SunMonTueWedThuFriSat" + (tp->tm_wday%7)*3),
+                       &"SunMonTueWedThuFriSat"[(tp->tm_wday%7)*3],
                        tp->tm_mday,
-                       ("JanFebMarAprMayJunJulAugSepOctNovDec"
-                        + (tp->tm_mon%12)*3),
+                       &"JanFebMarAprMayJunJulAugSepOctNovDec"
+                       [(tp->tm_mon%12)*3],
                        tp->tm_year + 1900,
                        tp->tm_hour,
                        tp->tm_min,
