@@ -140,7 +140,10 @@ change_pin (int unblock_v2, int allow_admin)
 	answer = cpr_get("cardutil.change_pin.menu",_("Your selection? "));
 	cpr_kill_prompt();
 	if (strlen (answer) != 1)
-	  continue;
+          {
+            xfree (answer);
+            continue;
+          }
 
 	if (*answer == '1')
 	  {
@@ -185,8 +188,10 @@ change_pin (int unblock_v2, int allow_admin)
 	  }
 	else if (*answer == 'q' || *answer == 'Q')
 	  {
+            xfree (answer);
 	    break;
 	  }
+        xfree (answer);
       }
 
   agent_release_card_info (&info);
@@ -343,6 +348,26 @@ fpr_is_ff (const char *fpr, unsigned int fprlen)
 }
 
 
+static void
+print_a_version (estream_t fp, const char *prefix, unsigned int value)
+{
+  unsigned int a, b, c, d;
+  a = ((value >> 24) & 0xff);
+  b = ((value >> 16) & 0xff);
+  c = ((value >>  8) & 0xff);
+  d = ((value      ) & 0xff);
+
+  if (a)
+    tty_fprintf (fp, "%s %u.%u.%u.%u\n", prefix, a, b, c, d);
+  else if (b)
+    tty_fprintf (fp, "%s %u.%u.%u\n", prefix, b, c, d);
+  else if (c)
+    tty_fprintf (fp, "%s %u.%u\n", prefix, c, d);
+  else
+    tty_fprintf (fp, "%s %u\n", prefix, d);
+}
+
+
 /* Print all available information about the current card. */
 static void
 current_card_status (ctrl_t ctrl, estream_t fp,
@@ -385,29 +410,29 @@ current_card_status (ctrl_t ctrl, estream_t fp,
       || strlen (info.serialno) != 32 )
     {
       const char *name1, *name2;
-      if (info.apptype && !strcmp (info.apptype, "openpgp"))
+      if (info.apptype && !ascii_strcasecmp (info.apptype, "openpgp"))
         goto openpgp;
-      else if (info.apptype && !strcmp (info.apptype, "NKS"))
+      else if (info.apptype && !ascii_strcasecmp (info.apptype, "NKS"))
         {
           name1 = "netkey";
           name2 = "NetKey";
         }
-      else if (info.apptype && !strcmp (info.apptype, "DINSIG"))
+      else if (info.apptype && !ascii_strcasecmp (info.apptype, "DINSIG"))
         {
           name1 = "dinsig";
           name2 = "DINSIG";
         }
-      else if (info.apptype && !strcmp (info.apptype, "P15"))
+      else if (info.apptype && !ascii_strcasecmp (info.apptype, "P15"))
         {
           name1 = "pkcs15";
           name2 = "PKCS#15";
         }
-      else if (info.apptype && !strcmp (info.apptype, "GELDKARTE"))
+      else if (info.apptype && !ascii_strcasecmp (info.apptype, "GELDKARTE"))
         {
           name1 = "geldkarte";
           name2 = "Geldkarte";
         }
-      else if (info.apptype && !strcmp (info.apptype, "PIV"))
+      else if (info.apptype && !ascii_strcasecmp (info.apptype, "PIV"))
         {
           name1 = "piv";
           name2 = "PIV";
@@ -443,7 +468,12 @@ current_card_status (ctrl_t ctrl, estream_t fp,
 
   if (opt.with_colons)
     {
-      es_fprintf (fp, "version:%.4s:\n", info.serialno+12);
+      if (info.appversion)
+        es_fprintf (fp, "version:%02u%02u:\n",
+                    (info.appversion >> 8) & 0xff,
+                    info.appversion & 0xff);
+      else
+        es_fprintf (fp, "version:%.4s:\n", info.serialno+12);
       uval = xtoi_2(info.serialno+16)*256 + xtoi_2 (info.serialno+18);
       pesc = (info.manufacturer_name
               ? percent_escape (info.manufacturer_name, NULL) : NULL);
@@ -543,7 +573,10 @@ current_card_status (ctrl_t ctrl, estream_t fp,
     }
   else
     {
-      tty_fprintf (fp, "Version ..........: %.1s%c.%.1s%c\n",
+      if (info.appversion)
+        print_a_version (fp, "Version ..........:", info.appversion);
+      else
+        tty_fprintf (fp, "Version ..........: %.1s%c.%.1s%c\n",
                    info.serialno[12] == '0'?"":info.serialno+12,
                    info.serialno[13],
                    info.serialno[14] == '0'?"":info.serialno+14,
@@ -782,7 +815,8 @@ static int
 change_name (void)
 {
   char *surname = NULL, *givenname = NULL;
-  char *isoname, *p;
+  char *isoname = NULL;
+  char *p;
   int rc;
 
   surname = get_one_name ("keygen.smartcard.surname",
@@ -793,7 +827,8 @@ change_name (void)
     {
       xfree (surname);
       xfree (givenname);
-      return -1; /*canceled*/
+      rc = gpg_error (GPG_ERR_CANCELED);
+      goto leave;
     }
 
   isoname = xmalloc ( strlen (surname) + 2 + strlen (givenname) + 1);
@@ -809,14 +844,17 @@ change_name (void)
       tty_printf (_("Error: Combined name too long "
                     "(limit is %d characters).\n"), 39);
       xfree (isoname);
-      return -1;
+      rc = gpg_error (GPG_ERR_TOO_LARGE);
+      goto leave;
     }
 
   rc = agent_scd_setattr ("DISP-NAME", isoname, strlen (isoname));
   if (rc)
     log_error ("error setting Name: %s\n", gpg_strerror (rc));
 
+ leave:
   xfree (isoname);
+  write_sc_op_status (rc);
   return rc;
 }
 
@@ -1450,7 +1488,10 @@ ask_card_keyattr (int keyno, const struct key_attr *current)
       algo = *answer? atoi (answer) : 0;
 
       if (!*answer || algo == 1 || algo == 2)
-        break;
+        {
+          xfree (answer);
+          break;
+        }
       else
         tty_printf (_("Invalid selection.\n"));
     }
