@@ -87,7 +87,7 @@ struct weakhash
 extern int g10_errors_seen;
 
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5 )
-  void g10_exit(int rc) __attribute__ ((noreturn));
+  void g10_exit(int rc) __attribute__ ((__noreturn__));
 #else
   void g10_exit(int rc);
 #endif
@@ -126,6 +126,8 @@ enum gcry_cipher_algos map_cipher_openpgp_to_gcry (cipher_algo_t algo);
 int openpgp_cipher_blocklen (cipher_algo_t algo);
 int openpgp_cipher_test_algo(cipher_algo_t algo);
 const char *openpgp_cipher_algo_name (cipher_algo_t algo);
+const char *openpgp_cipher_algo_mode_name (cipher_algo_t algo,
+                                           aead_algo_t aead);
 
 gpg_error_t openpgp_aead_test_algo (aead_algo_t algo);
 const char *openpgp_aead_algo_name (aead_algo_t algo);
@@ -443,6 +445,7 @@ gpg_error_t write_keyblock_to_output (kbnode_t keyblock,
                                       int with_armor, unsigned int options);
 
 gpg_error_t export_ssh_key (ctrl_t ctrl, const char *userid);
+gpg_error_t export_secret_ssh_key (ctrl_t ctrl, const char *userid);
 
 /*-- dearmor.c --*/
 int dearmor_file( const char *fname );
